/*
Capture patch <http://quakecapture.sourceforge.net/>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "ScreenShotWriter.h"

#include <stdio.h>
#include <string.h>

namespace Capture {
    extern void SetError_s(char*, char*);
}

using namespace Capture;

ScreenShotWriter::ScreenShotWriter(char* codec00, float fps, int audio_hz)
 :  IWriter(codec00, fps, audio_hz),
    m_time(0.0)
{
}

ScreenShotWriter::~ScreenShotWriter()
{
}

bool ScreenShotWriter::Open(char* filename)
{
    strcpy(m_basefilename, filename);

    if (m_codec00 == NULL) m_codec00 = "tga";

    if (strcmp(m_codec00, "tga")) {
        SetError_s("capture mode screenshot: codec '%s' not recognised\n", m_codec00);
        return false;
    }

    return true;
}

bool ScreenShotWriter::WriteVideo(int width, int height, unsigned char* pixel_buffer)
{
    float seconds = m_time;
    m_time += 1.0 / m_fps;
    int minutes = (int)(seconds / 60); seconds -= 60 * minutes;
    int hours   = minutes / 60; minutes = minutes % 60;

    char filename[256];
    sprintf(filename, "%s-%d-%02d-%05.2f.%s", m_basefilename, hours, minutes, seconds, m_codec00);

	char header[18];
	memset(header, 0, 18);
	header[ 2] = 2;		// uncompressed type
	header[12] = width  &  255;
	header[13] = width  >> 8;
	header[14] = height &  255;
	header[15] = height >> 8;
	header[16] = 24;	// pixel size

    FILE* file = fopen(filename, "wb");
    if (file == NULL) return false;

    bool ok = (fwrite(header, 1, 18, file) == 18);
    int size = 3 * width * height;
    ok = ok && (fwrite(pixel_buffer, 1, size, file) == size);

    fclose(file);

    return ok;
}

bool ScreenShotWriter::WriteAudio(int samples, unsigned char* sample_buffer)
{
    // No audio for this capture type, we silently do nothing.
    return true;
}
