/*
Capture patch <http://quakecapture.sourceforge.net/>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

// This file contains the Quake1-specific parts of the capture patch

#include "helpers.h"
#include "../client/client.h"
#include "../client/snd_loc.h"
extern void SV_Map (qboolean attractloop, char *levelstring, qboolean loadgame);
extern cvar_t *Cvar_FindVar (char *var_name);

extern cvar_t* fixedtime;
extern dma_t dma;
extern float scr_con_current;
extern int scr_draw_loading;


void CaptureHelper_Printf_f(char* format, float float_arg)
{
    Com_Printf(format, float_arg);
}

void CaptureHelper_Printf_s(char* format, char* string_arg)
{
    Com_Printf(format, string_arg);
}

void CaptureHelper_Print(char* string)
{
    Com_Printf("%s", string);
}

float CaptureHelper_VideoFps(void)
{
    if (fixedtime->value) return 1000.0f / fixedtime->value;
    else return 0.0;
}

void CaptureHelper_SetVideoFps(float fps)
{
    qboolean cl_minfps_exists = strcmp(Cvar_VariableString("cl_minfps"), "");
    float minfps = cl_minfps_exists ? Cvar_VariableValue("cl_minfps") : 5.0;
    if (fps == 0.0) {
        Cvar_SetValue("fixedtime", 0.0);
        return;
    }
    Cvar_SetValue("fixedtime", (int)(0.5 + 1000.0 / fps));
    if (Cvar_VariableValue("s_mixahead") < 1.0 / fps) {
        CaptureHelper_Print("Warning: you may want to increase s_mixahead!\n");
    }
    if (Cvar_VariableValue("cl_maxfps") < fps) {
        CaptureHelper_Print("Warning: you may need to increase cl_maxfps!\n");
    }
    if (minfps > fps) {
        if (cl_minfps_exists) {
            CaptureHelper_Print("Warning: you may need to decrease cl_minfps!\n");
        } else {
            CaptureHelper_Print("Warning: fps set below the usual hard coded minimum of 5. ");
            CaptureHelper_Print("You may need to re-enable cl_minfps in source code!\n");
        }
    }
    Cvar_SetValue("cheats", 1); // prevent fixedtime being reset in multi-player demos
}


int CaptureHelper_AudioHz(void)
{
    return dma.speed;
}

qboolean CaptureHelper_ShouldSuppressCapture(void)
{
    return 
        (cls.state != ca_active) ||
        !cl.refresh_prepped ||
        // don't output whilst console is down or 'loading' is displayed
        (scr_con_current > 0) ||
        (scr_draw_loading);
}

qboolean CaptureHelper_CopyVideo(unsigned char* buffer)
{
    char command_text[80];

    // We make a copy the GL pixel buffer via a backdoor, using a console command.
    // Using this backdoor means we avoid having to change the rendering API,
    //  which makes the patch easier to integrate with existing engine ports;
    //  we will sacrifice proper type-checking etc. in favour of this easiness.
    if (Cmd_Exists("gl_copypixels_backdoor")) {
        sprintf(command_text, "gl_copypixels_backdoor %u %u %u\0", viddef.width, viddef.height, (void*)buffer);
        Cmd_ExecuteString(command_text);
        return true;
    } else {
        CaptureHelper_Print("No gl_copypixels_backdoor found.\n");
        CaptureHelper_Print("Use GL rendering, check ref_gl.dll version.\n");
        return false;
    }
}

qboolean CaptureHelper_DemoExists(char* demoname)
{
    char fullname[MAX_QPATH];
    strcpy(fullname, "demos/");
    strcpy(fullname + 6, demoname);
    return (FS_LoadFile(fullname, NULL) >= 0);
}

qboolean CaptureHelper_PlayDemo_f(void)
{
    char demoname[256];
    strcpy(demoname, Cmd_Argv(1));
    COM_DefaultExtension(demoname, CaptureHelper_DefaultDemoExtension());

    SV_Map(true, demoname, false);
    return CaptureHelper_DemoExists(demoname);
}

void CaptureHelper_SetBatchCapture(qboolean batch)
{
    cls.capturedemo = batch;
}

qboolean CaptureHelper_BatchCapture(void)
{
    return cls.capturedemo;
}

int CaptureHelper_VideoWidth(void)
{
    return viddef.width;
}

int CaptureHelper_VideoHeight(void)
{
    return viddef.height;
}

int CaptureHelper_SoundChannels(void)
{
    return dma.channels;
}

char* CaptureHelper_DefaultDemoExtension(void)
{
    return ".dm2";
}

void CaptureHelper_InitVariable(char* variablename, char* defaultvalue)
{
    Cvar_Get(variablename, defaultvalue, CVAR_USERINFO | CVAR_ARCHIVE);

}
void CaptureHelper_Shutdown(void)
{
    SV_Shutdown ("Automatic exit on capture_demo disconnect\n", false);
	Sys_Quit();
}

